Package: CHRONOS
Version: 1.38.0
Date: 2020-09-05
Title: CHRONOS: A time-varying method for microRNA-mediated sub-pathway
        enrichment analysis
Author: Aristidis G. Vrahatis, Konstantina Dimitrakopoulou, Panos
        Balomenos
Maintainer: Panos Balomenos <balomenos@upatras.gr>
Description: A package used for efficient unraveling of the inherent
        dynamic properties of pathways. MicroRNA-mediated subpathway
        topologies are extracted and evaluated by exploiting the
        temporal transition and the fold change activity of the linked
        genes/microRNAs.
Depends: R (>= 3.5)
SystemRequirements: Java version >= 1.7, Pandoc
License: GPL-2
NeedsCompilation: no
LazyLoad: yes
Imports: XML, RCurl, RBGL, parallel, foreach, doParallel, openxlsx,
        igraph, circlize, graph, stats, utils, grDevices, graphics,
        methods, biomaRt, rJava
Suggests: RUnit, BiocGenerics, knitr, rmarkdown
VignetteBuilder: knitr
biocViews: SystemsBiology, GraphAndNetwork, Pathways, KEGG
git_url: https://git.bioconductor.org/packages/CHRONOS
git_branch: RELEASE_3_22
git_last_commit: 6ae799a
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Packaged: 2025-10-30 03:07:55 UTC; biocbuild
